/*
 * Copyright 2008-2010 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wamblee.glassfish.statistics;

import java.lang.management.ManagementFactory;
import java.util.Hashtable;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.management.JMException;
import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.wamblee.glassfish.repo.AuthenticationRepository;
import org.wamblee.glassfish.statistics.Statistic.Clock;
import org.wamblee.glassfish.statistics.Statistic.Measurement;

public class StatisticsAuthenticationRepository implements AuthenticationRepository, StatisticsAuthenticationRepositoryMXBean {
    private static final Logger LOGGER = Logger.getLogger(StatisticsAuthenticationRepository.class.getName());

    public static final String JMX_REALM_PROPERTY = "realm";
    public static final String JMX_REPO_PROPERTY = "repo"; 
    public static final String JMX_DOMAIN = "org.wamblee.glassfish.auth.FlexibleJdbcRealm";
    
    private Statistic passwordStatistic;
    private Statistic seedStatistic;
    private Statistic groupStatistic;
    
    private String realm;
    private String repoName;
    private AuthenticationRepository repository; 
    
    public StatisticsAuthenticationRepository(Clock aClock, String aRealm, String aRepoName, AuthenticationRepository aRepository) {
        passwordStatistic = new Statistic(aClock);
        seedStatistic = new Statistic(aClock);
        groupStatistic = new Statistic(aClock);

        realm = aRealm;
        repoName = aRepoName;
        repository = aRepository;
        register();
    }
    
    private void register() { 
        MBeanServer server = ManagementFactory.getPlatformMBeanServer();
        try {
            Hashtable<String, String> props = new Hashtable<String, String>();
            props.put(JMX_REALM_PROPERTY, realm);
            props.put(JMX_REPO_PROPERTY, repoName);
            ObjectName name = new ObjectName(JMX_DOMAIN, props);
            try {
                server.unregisterMBean(name);
            } catch (JMException e) {
                // ignore. We need to do this because realms can be deleted at
                // runtime but there
                // is no callback for this so we cannot unregister. Therefore we
                // try
                // to unregister
                // when the realm is started.
            }
            server.registerMBean(this, name);
            LOGGER.info("Registered MBean at '" + name + "'");
        } catch (JMException e) {
            LOGGER.log(Level.WARNING,
                "Could not register MBean for SimpleExpiryCache for realm '" +
                    realm + "' and repository '" + repoName + "'", e);
        }
    }
    
    @Override
    public String findPassword(String aUsername) {
        Measurement m = passwordStatistic.start();
        String password = null; 
        try { 
           password = repository.findPassword(aUsername);
        } finally { 
           m.stop(password != null);
        }
        return password;
    }
    
    @Override
    public String findSeed(String aUsername) {
        Measurement m = seedStatistic.start();
        String seed = null; 
        try { 
           seed = repository.findSeed(aUsername);
        } finally { 
            m.stop(seed != null);
        }
        return seed; 
    }
    
    @Override
    public List<String> findGroups(String aUsername) {
        Measurement m = groupStatistic.start();
        List<String> groups = null;
        try { 
           groups = repository.findGroups(aUsername);
        } finally { 
            m.stop(groups != null && groups.size() > 0 );
        }
        return groups;
    }
    
    /* (non-Javadoc)
     * @see org.wamblee.glassfish.statistics.StatisticsAuthenticationRepositoryMXBean#getPasswordStatistic()
     */
    @Override
    public Statistic getPasswordStatistic() {
        return passwordStatistic;
    }
    
    /* (non-Javadoc)
     * @see org.wamblee.glassfish.statistics.StatisticsAuthenticationRepositoryMXBean#getSeedStatistic()
     */
    @Override
    public Statistic getSeedStatistic() {
        return seedStatistic;
    }
    
    /* (non-Javadoc)
     * @see org.wamblee.glassfish.statistics.StatisticsAuthenticationRepositoryMXBean#getGroupStatistic()
     */
    @Override
    public Statistic getGroupStatistic() {
        return groupStatistic;
    }
    
    @Override
    public void clearStatistics() {
        passwordStatistic.clear();
        seedStatistic.clear();
        groupStatistic.clear();
    }
}
